from typing import List

from core.verify import SigningService
from avvyprotocol.l3_pb2 import TxnResponseBody, MsgResponse, TxnRequest, TxnResponse


class ResponseConstructor:
    def __init__(self, signing_service: SigningService, request: TxnRequest):
        self.req = request
        self.responses = []
        self.signing_service = signing_service

    def make_redirect(self, redirect_uri):
        msg_id = self.req.body.requests[0].id
        self.responses.append(
            MsgResponse(
                msg_id=msg_id,
                status=MsgResponse.Status.REDIRECT,
                body={'redirect_uri': redirect_uri.encode()}
            )
        )
        return self.make_tx_response()

    def make_error(self, error_info):
        msg_id = self.req.body.requests[0].id
        self.responses.append(
            MsgResponse(
                msg_id=msg_id,
                status=MsgResponse.Status.DECLINED,
                body={'error': error_info.encode()}
            )
        )
        return self.make_tx_response()

    def make_success(self):
        msg_id = self.req.body.requests[0].id
        self.responses.append(
            MsgResponse(
                msg_id=msg_id,
                status=MsgResponse.Status.ACCEPTED,
                body={}
            )
        )
        return self.make_tx_response()

    def set_response_messages(self, msg_response: List[MsgResponse]):
        self.responses = msg_response

    def make_tx_response(self):
        body = TxnResponseBody(
            txn_id=self.req.body.txn_id,
            responses=self.responses
        )
        return TxnResponse(
            body=body,
            auth_info=self.signing_service.get_auth_info(body)
        )
