import os
import shutil

from ipfshttpclient import Client, connect

from config import IPFS_CONNECTION_URL, FILE_TMP_STORAGE
from core.utils import init_log, Tryings

log = init_log()


class IpfsDataStore:
    """
    Подключение к ипфс с возможностью добавления и извлечения данных.
    """

    def __init__(self):
        with Tryings(5, connect, IPFS_CONNECTION_URL) as conn:
            self.client: Client = conn


class MerkleRootStore(IpfsDataStore):
    """
    Сохранение в ипфс и загрузка директории с merkle root
    """

    def put_snapshot(self, file_path) -> bytes:
        """
        Добавление директории от меркл стораджа
        Возвращается ключ от json в котором хранится хэш директории и ее имя
        """
        result = self.client.add(file_path, recursive=True, pin=True)
        cid = result[-1]['Hash']
        return self.client.add_json({'name': file_path, 'cid': cid}).encode()

    def load_snapshot(self, key: bytes):
        """
        Загрузка директории по ключу.
        Из ключа получаем инфу об сиде и имени директории меркла.
        Выкачиваем во временное хранилище и переименовываем

        """
        file_data = self.client.get_json(key)
        if not file_data or not isinstance(file_data, dict):
            raise Exception(f'No data by key: {key}')

        try:
            self.client.get(cid=file_data['cid'], target=FILE_TMP_STORAGE)
            source = os.path.join(FILE_TMP_STORAGE, file_data['cid']) + '/'
            dest = file_data['name']
            shutil.rmtree(dest, ignore_errors=True)
            move_result = shutil.copytree(source, dest, dirs_exist_ok=True)
            return move_result
        except Exception as e:
            log.exception(e)


merkle_root_ipfs_store = MerkleRootStore()
