import grpc

from core.consensus import Consensus
from core.helpers import ResponseConstructor
from core.utils import init_log
from core.verify import SigningService, verify
from avvyprotocol.l3_pb2 import TxnRequest
from avvyprotocol.l3_pb2_grpc import L3ServiceServicer, add_L3ServiceServicer_to_server

from load_test.utils import set_timestamp

log = init_log('l3_node')


class L3Servicer(L3ServiceServicer):
    signing_service = SigningService()
    """
    Сервер грпс, обработка транзакции: верификация, редирект если не лидер и синхронизация транзакции
    """

    def __init__(self, consensus_proc: Consensus):
        self.consensus_proc = consensus_proc
        log.info(f'l3 servicer inited')

    async def ProcessTxn(self, txn: TxnRequest, context):
        rc = ResponseConstructor(signing_service=self.signing_service, request=txn)

        # todo fix it
        # verify_result = verify(txn.auth_info, txn.body)
        # if not verify_result:
        #     log.warn(f'Tx {txn.body.txn_id} not verified')
        #     return rc.make_error('not verified')

        redirect_uri = self.consensus_proc.get_leader_grpc_conn()
        if redirect_uri:
            return rc.make_redirect(redirect_uri)

        set_timestamp(txn, '_tst_grpc_receive')
        self.consensus_proc.sync_transaction(txn)

        return rc.make_success()


async def start_grpc_server(consensus_processor: Consensus, port):
    address = f'0.0.0.0:{port}'
    servicer = L3Servicer(consensus_processor)

    server = grpc.aio.server(compression=grpc.Compression.Gzip)
    add_L3ServiceServicer_to_server(servicer, server)

    server.add_insecure_port(address)
    await server.start()

    log.info(f'GRPC Server [protobuf] started at: {address}')
    try:
        await server.wait_for_termination()
    except Exception as e:
        log.exception(e)
    finally:
        log.info(f'GRPC Server [protobuf] stopped')
