import time

from pymongo import MongoClient


class MakeTestStore:
    def __init__(self):
        self.client = MongoClient()
        self.db = self.client['l3-load-test']
        # self.db.drop_collection('msgs')
        self.collection = self.db.get_collection('msgs')
        if self.collection is None:
            self.collection = self.db.create_collection('msgs')
            self.collection.create_index([('client_marker',)])

    def add_processed_msg(self, client_marker, msg_id, msg_body):
        tst_data = {k: float(v) for k, v in dict(msg_body).items() if k.startswith('_tst_')}
        tst_data['_tst_client_receive'] = time.time()
        self.collection.insert_one({'client_marker': client_marker, 'msg_id': msg_id, **tst_data})


class ReadTestStore:
    def __init__(self):
        self.client = MongoClient()
        self.db = self.client['l3-load-test']
        self.collection = self.db.get_collection('msgs')
        assert self.collection is not None

    def get_by_clients_markers(self, start_name='_tst_client_send', finish_name='_tst_client_receive'):
        result = self.collection.aggregate(
            [{'$project': {'client_marker': 1, 'time': {'$subtract': [f"${finish_name}", f"${start_name}"]}}},
             {
                 '$group': {'_id': '$client_marker',
                            'avg_time': {'$avg': '$time'},
                            'std_time': {'$stdDevPop': '$time'},
                            'msg_count': {'$count': {}}
                            # 'all_times': {'$push': '$time'}
                            }
             },
             ]
        )
        return {r['_id']: r for r in result}


if __name__ == '__main__':
    rs = ReadTestStore()
    result = rs.get_by_clients_markers()
    for cl_marker, info in result.items():
        assert '_id' in info
        assert 'avg_time' in info
        assert 'std_time' in info
        # assert 'all_times' in info
        # np_std = np.std(info['all_times'])
