extern crate merk;

use merk::*;
use pyo3::prelude::*;
use pyo3::wrap_pyfunction;

#[pyfunction]
fn create_db(path: &str) -> PyResult<()> {
    Merk::open(path).unwrap();
    Ok(())
}

#[pyfunction]
fn apply(path: &str, key1: &str, value1: &str) -> PyResult<()> {
    let mut merk = Merk::open(path).unwrap();

    let batch = [(
        key1.as_bytes().to_vec(),
        Op::Put(value1.as_bytes().to_vec()),
    )];

    merk.apply(&batch, &[]).unwrap();
    Ok(())
}

#[pyfunction]
fn apply_batch(path: &str, keys: Vec<&str>, values: Vec<&str>) -> PyResult<()> {
    let mut merk = Merk::open(path).unwrap();

    let mut batchs: Vec<(Vec<u8>, Op)> = Vec::new();
    let mut batch: (Vec<u8>, Op);

    for (index, _key) in keys.iter().enumerate() {
        batch = (
            keys[index].as_bytes().to_vec(),
            Op::Put(values[index].as_bytes().to_vec()),
        );
        batchs.push(batch);
    }

    merk.apply(&batchs, &[]).unwrap();
    Ok(())
}

#[pyfunction]
fn get(path: &str, key: &str) -> PyResult<Vec<u8>> {
    let merk = Merk::open(path).expect("failed to open merk");

    Ok(merk.get(key.as_bytes()).unwrap().unwrap())
}

#[pyfunction]
fn root_hash(path: &str) -> PyResult<([u8; 32])> {
    let merk = Merk::open(path).expect("failed to open merk");

    Ok(merk.root_hash())
}

#[pymodule]
fn pymerk(_py: Python<'_>, m: &PyModule) -> PyResult<()> {
    m.add_wrapped(wrap_pyfunction!(create_db)).unwrap();
    m.add_wrapped(wrap_pyfunction!(apply)).unwrap();
    m.add_wrapped(wrap_pyfunction!(apply_batch)).unwrap();
    m.add_wrapped(wrap_pyfunction!(get)).unwrap();
    m.add_wrapped(wrap_pyfunction!(root_hash)).unwrap();
    Ok(())
}
