import unittest
import uuid

from avvyprotocol.l2_pb2 import Rollup

from config import IIDAP_ID
from core.startup import StartEngine
from core.store.data import merkle_root_ipfs_store
from core.store.prove import Storage
from core.utils import generate_id
from l2.subproc import SubProcL2Client


class StartupTestCase(unittest.IsolatedAsyncioTestCase):
    async def test_smoke(self):
        storage = Storage(IIDAP_ID)
        k1, k2 = uuid.uuid4().hex, uuid.uuid4().hex

        l2_client = SubProcL2Client()

        storage.add_dict({k1: 'test'})

        ds_uri = merkle_root_ipfs_store.put_snapshot(storage.file_path)
        rollup = Rollup(id=generate_id(),
                        prev_hash=b'prev',
                        current_hash=b'curr',
                        txns_list=[],
                        data_source_uri=ds_uri)

        storage.add_dict({k2: 'test 2'})

        digest_id = l2_client.send_rollup(rollup)
        l2_client.set_rollup_digest_status(digest_id, rollup, 'ACCEPTED')

        startup = StartEngine(loop=self._asyncioTestLoop)
        startup.start_iidap()

        exist = storage.get_raw(k1)
        # not_exists = storage.get_raw(k2)
        self.assertIsNotNone(exist)
        # self.assertIsNone(not_exists)
