import unittest

from avvyprotocol.l2_pb2 import Rollup

from core.rollup import RollupDigestProcessor
from core.utils import generate_id
from l2.subproc import SubProcL2Client
from tests.rollup_test import FakeEventListeners, FakeStartEngine


class SubProcL2ClientTest(unittest.IsolatedAsyncioTestCase):
    async def test_smoke(self):
        event_listeners = FakeEventListeners()
        start_engine = FakeStartEngine()
        # prepare
        rollup_digest_proc = RollupDigestProcessor(event_listeners, start_engine)

        l2client = SubProcL2Client()
        reg_res = l2client.l3_register()
        self.assertIsNotNone(reg_res)
        rollup = Rollup(id=generate_id(),
                        prev_hash=b'prev',
                        current_hash=b'curr',
                        txns_list=[],
                        data_source_uri=b'source_uri')
        digest_id = await l2client.send_rollup_async(rollup)
        self.assertIsNotNone(digest_id)

        l2client.set_rollup_digest_status(digest_id, rollup, 'ACCEPTED')
        digest = l2client.get_last_accepted_rollup_digest()
        self.assertIsNotNone(digest)

        rollup_digest_proc.stop_task()
