import asyncio
from unittest.async_case import IsolatedAsyncioTestCase

from client.generators import TimedTxnManager, default_subscribe_filters, make_subscribe_msg
from core.consensus import Consensus
from core.rollup import RollupProcessor
from core.subscribe.event_listeners import EventListeners
from core.transactions_processing import TxnsProcessor
from core.utils import init_log
from core.verify import SigningService
from avvyprotocol.l3_pb2 import TxnRequest
from tests.subscribe_test import FakeClientConnection

log = init_log(only_stream=True)


class FakeConsensus(Consensus):
    def __init__(self, txns_proc: TxnsProcessor):
        super().__init__(txns_proc)

    def get_leader_grpc_conn(self):
        return None


class FakeRollupProc():
    def __init__(self, ):
        self.appended_txns = []
        self.finished = False

    def append_txn(self, txn: TxnRequest):
        self.appended_txns.append(txn)

    def at_finish(self):
        self.finished = True


class TxnProcessTest(IsolatedAsyncioTestCase):
    def setUp(self) -> None:
        self.event_listeners = EventListeners()
        self.txns_proc = TxnsProcessor(asyncio.get_event_loop(), event_listeners=self.event_listeners)
        self.txns_proc.set_rollup_processor(FakeRollupProc())

        self.signing_service = SigningService('test_key')
        self.ttm = TimedTxnManager(self.signing_service)

    async def test_txn_processing(self):
        client = FakeClientConnection()
        self.event_listeners.set_client_connection(self.signing_service.pub.decode(), client)
        for txn in self.ttm.generate_txs(1, 10):
            await self.txns_proc.process_txn(txn)

        assert len(client.messages) == 10

        for txn in self.ttm.generate_txs(1, 1):
            await self.txns_proc.process_txn(txn)

        assert len(client.messages) == 11

        for txn in self.ttm.generate_txs(1, 1, data=default_subscribe_filters, msg_factory=make_subscribe_msg):
            await self.txns_proc.process_txn(txn)

        assert len(client.messages) == 12
