import json
import unittest

from core.utils import generate_id
from core.verify import prepare_keys, sign_data, dump_pub, load_pub, SigningService, verify
from avvyprotocol.l3_pb2 import TxnRequestBody, Msg


class VerifyTestCase(unittest.TestCase):
    def test_instruments(self):
        priv, pub = prepare_keys()
        assert priv
        assert pub

        data = json.dumps({'test': {'test': ['test']}})
        sign = sign_data(priv, data)
        assert sign
        assert isinstance(sign, bytes)

        dumped_pub = dump_pub(pub)
        assert dumped_pub
        assert isinstance(dumped_pub, bytes)

        loaded_pub = load_pub(dumped_pub)
        assert loaded_pub == pub

    def test_keeper(self):
        keeper1 = SigningService('test_key1')
        keeper2 = SigningService('test_key2')
        assert keeper1.pub
        assert isinstance(keeper1.pub, bytes)

        body1 = TxnRequestBody(
            txn_id=generate_id(),
            requests=[Msg(
                id=generate_id(),
                body={'foo1': b'bar1'}
            )]
        )
        body2 = TxnRequestBody(
            txn_id=generate_id(),
            requests=[Msg(
                id=generate_id(),
                body={'foo2': b'bar2'}
            )]
        )
        auth_info1 = keeper1.get_auth_info(body1)
        auth_info2 = keeper2.get_auth_info(body2)

        assert verify(auth_info1, body1)
        assert verify(auth_info2, body2)

        assert not verify(auth_info1, body2)
        assert not verify(auth_info2, body1)
