from aiohttp import web

from core.subscribe.event_listeners import EventListeners
from core.utils import init_log
from ws.connection import ClientConnection

DEFAULT_PORT = 54632

log = init_log('ws')

routes = web.RouteTableDef()


@routes.get('/ws/{public_key}')
async def ws_endpoint(request):
    client_key = request.match_info['public_key']
    log.info(f'Start ws connection from {request.remote}, client: {client_key}')
    ws = web.WebSocketResponse()
    await ws.prepare(request)

    conn = ClientConnection(ws)
    request.app['event_listeners'].set_client_connection(client_key, conn)
    try:
        await conn.handle_connection()
    except Exception as e:
        log.exception(e)
    finally:
        await request.app['event_listeners'].close_client_connection(client_key)

    log.info(f'Finish handle ws connection from {request.remote}, client: {client_key}')
    return ws


def init_ws_app(event_listeners: EventListeners):
    app = web.Application()
    app.add_routes(routes)

    app['event_listeners'] = event_listeners

    async def on_shutdown(app):
        log.info(f'Shutdown ws app')
        await app['event_listeners'].close_all_connections()

    app.on_shutdown.append(on_shutdown)
    return app


async def start_ws_server(port, event_listeners: EventListeners):
    runner = web.AppRunner(init_ws_app(event_listeners))
    await runner.setup()
    site = web.TCPSite(runner, '0.0.0.0', port)
    await site.start()
    log.info(f'Start ws server at ::{port}')
