package main

import (
	"encoding/json"
	"fmt"
	"log"
	"os"

	"google.golang.org/grpc"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"

	l3_grpc "gitlab.domi.do/avvy/avvy-l3/l3_node_golang/api/grpc"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/compiler"
)

var example1 = `{
    name: "world";
    events: [
        {
            msg: $concat {a: "hello, "; b: ~name;};
        }
    ];
}`

var example2 = `{
    prefix: "hello, ",
    name: "world";
    make_hello: $concat {a: ~prefix; b: @name;}
    events: [
        {
            msg: ~make_hello {name: ..name;};
        }
    ];
}`

func input() (block []*aproto.Prop, err error) {

	var (
		buf string
	)

	for {
		var line string
		fmt.Scan(&line)
		switch line {
		case "#show":
			fmt.Println("[buffer]:", buf)
		case "#clear":
			buf = ""
		case "#parse":
			if block, err = compiler.Compile(buf); err == nil {
				bts, _ := json.Marshal(block)
				fmt.Println("[block]:", string(bts))
			} else {
				fmt.Println("[error]:", err)
			}
		case "#exec":
			block, err = compiler.Compile(buf)
			return
		case "#exit":
			os.Exit(0)
		default:
			buf += " " + line
		}
	}

	return
}

func run() (err error) {

	client, err := l3_grpc.NewClient("127.0.0.1:80", grpc.WithInsecure())
	if err != nil {
		return
	}
	defer client.Close()

	stream, err := client.StartStream("repl", nil)
	if err != nil {
		return
	}
	defer stream.CloseSend()

	go func() {
		for {
			event, err := stream.Recv()
			if err != nil {
				fmt.Println("[error]:", err)
				break
			}
			bts, _ := json.Marshal(*event.Msg)
			fmt.Println("[event.msg]:", string(bts))
		}
	}()

	for {
		if block, err := input(); err == nil {
			txn := &aproto.TxnRequest{
				AuthInfo: &aproto.AuthInfo{
					Signers: []*aproto.Signer{
						{
							PublicKey: []byte("repl"),
						},
					},
				},
				Body: &aproto.TxnRequestBody{
					TxnId: 1,
					Requests: []*aproto.Msg{
						{
							Id:   1,
							Data: block,
						},
					},
				},
			}

			if err = stream.Send(txn); err != nil {
				fmt.Println("[error]:", err)
			}
		} else {
			fmt.Println("[error]:", err)
		}
	}

	return
}

func main() {
	err := run()
	if err != nil {
		log.Fatalln(err)
	}
}
