package object

import (
	"go.uber.org/zap"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	dsl "gitlab.domi.do/avvy/avvy-l3/l3_node_golang/old_dsl"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
)

var logger = utils.GetLogger("OBJECT")

type Object struct {
	ID      string
	Balance uint64 // Need bigint
	Props   *lang.Props
}

func ObjectFromProps(props []*aproto.Prop) (obj *Object) {
	obj = &Object{}

	for _, prop := range props {
		val := prop.GetValue()
		switch prop.GetName() {
		case "object_id":
			obj.ID = val.GetSingular().GetStringValue()
		case "balance":
			obj.Balance = val.GetSingular().GetUintValue()
		default:
			obj.Props = append(obj.Props, prop)
		}
	}

	return
}

func (obj *Object) New() *Object {
	return &Object{
		ID:      "123456",
		Balance: 0,
		Props:   obj.Props,
	}
}

func (obj *Object) ToProps() (props []*aproto.Prop) {
	props = make([]*aproto.Prop, 0, len(obj.Props)+2)

	props = append(props, &aproto.Prop{
		Name:  "object_id",
		Value: lang.StringPayload(obj.ID),
	})

	props = append(props, &aproto.Prop{
		Name:  "balance",
		Value: lang.UintPayload(obj.Balance),
	})

	props = append(props, obj.Props...)

	return
}

func (obj *Object) GetHandler(name string) *dsl.Contract {
	if handlerPayload, ok := obj.Props[name]; ok {
		if handler, err := dsl.Build(handlerPayload); err == nil {
			return handler
		}
	} else if handler, ok := DefaultHandlers[name]; ok {
		return handler
	}
	return nil
}

func (obj *Object) Verify(msg types.MsgBody) bool {
	handler := obj.GetHandler("verify")
	out, err := handler.Exec(utils.MapPayload(msg))
	if err != nil {
		logger.Error("OBJECT VERIFY ERROR", zap.Error(err))
		return false
	}
	return out.GetSingular().GetBoolValue()
}

func (obj *Object) HandleMsg(msg types.MsgBody) (err error) {
	// var out *aproto.Payload
	handlerType := msg["type"].GetSingular().GetStringValue()
	if handler := obj.GetHandler(handlerType); handler != nil {
		_, err = handler.Exec(utils.MapPayload(msg))
	}
	return
}
