package context

import "gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/interfaces"

type stackItem = interface {
	interfaces.Getter
	interfaces.Setter
}

type Context struct {
	heap  interfaces.Getter
	stack []stackItem

	depth   uint64
	isBreak bool
}

func (ctx *Context) copy() *Context {
	return &Context{
		heap:  ctx.heap,
		stack: ctx.stack,

		depth:   ctx.depth,
		isBreak: ctx.isBreak,
	}
}

func NewContext(heap interfaces.Getter) (ctx *Context) {
	ctx = &Context{
		stack: make([]stackItem, 0),
	}
	return
}

func (ctx *Context) WithBreak() interfaces.Context {
	copy := ctx.copy()
	copy.isBreak = true
	return copy
}

func (ctx *Context) IsBreak() bool {
	return ctx.isBreak
}

func (ctx *Context) Step() {
	ctx.depth++
}

// STACK BACK GET/SET

func (ctx *Context) Get(key string) (val interfaces.Unit) {
	if n := len(ctx.stack) - 1; n != -1 {
		val = ctx.stack[n].Get(key)
	}
	if val == nil {
		ctx.heap.Get(key)
	}
	return
}

func (ctx *Context) Set(key string, val interfaces.Unit) {
	if n := len(ctx.stack) - 1; n != -1 {
		ctx.stack[n].Set(key, val)
	}
}

// STACK
func (ctx *Context) Push(val stackItem) {
	ctx.stack = append(ctx.stack, val)
}

func (ctx *Context) Pop() (val interfaces.Getter) {
	n := len(ctx.stack) - 1
	if n == -1 {
		return
	}
	val = (ctx.stack)[n]
	ctx.stack = (ctx.stack)[:n]
	return
}

func (ctx *Context) Back() (val stackItem) {
	if n := len(ctx.stack) - 1; n != -1 {
		val = ctx.stack[n]
	}
	return
}
