package core

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/interfaces"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/typing"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

type functorBlock interface {
	interfaces.Getter
	interfaces.Keyable
	interfaces.Sized
	interfaces.Setter
	interfaces.Evaluable
	interfaces.Typable
}

type Functor struct {
	name   string
	input  functorBlock
	nested functorBlock
	nodes  functorBlock
	output functorBlock
}

func BuildFunctor(payload *aproto.Payload, ctx interfaces.Context) (fn *Functor, err error) {

	funcPayload := payload.GetFunctor()
	if funcPayload == nil {
		err = errors.BAD_PAYLOAD_ERR
		return
	}

	fn = &Functor{
		name: funcPayload.GetName(),
	}

	env := NewProps(nil)

	buildBlock := func(props []*aproto.Prop) (functorBlock, error) {
		block := NewProps(nil)
		for _, prop := range props {
			if prop, err := BuildProp(prop, ctx); err == nil {
				block.Set(prop.Name, prop)
				env.Set(prop.Name, prop)
			} else {
				return nil, err
			}
		}
		return block, nil
	}

	if fn.nested, err = buildBlock(funcPayload.GetNested()); err == nil {
		if fn.input, err = buildBlock(funcPayload.Input); err == nil {
			ctx.Push(env)
			defer ctx.Pop()

			if fn.nodes, err = buildBlock(funcPayload.GetNodes()); err == nil {
				fn.output, err = buildBlock(funcPayload.GetOutput())
			}
		}
	}

	return
}

func (fn *Functor) InputSpec() interface {
	interfaces.Getter
	interfaces.Keyable
	interfaces.Sized
	interfaces.Typable
} {
	return fn.input
}

// Impl interfaces.Getter
func (fn *Functor) Get(name string) (unit interfaces.Unit) {
	return fn.nested.Get(name)
}

// Impl interfaces.Evaluable
func (fn *Functor) Eval(ctx interfaces.Context) (interfaces.Unit, error) {
	for _, key := range fn.nested.Keys() {
		ctx.Set(key, fn.nested.Get(key))
	}

	return fn.output.Eval(ctx)
}

// Impl interfaces.Typable
func (fn *Functor) Type(ctx interfaces.Context) typing.Type {
	return fn.output.Type(ctx)
}
