package core

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/interfaces"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/typing"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

type Prop struct {
	Name  string
	Value interfaces.Unit
	Spec  *Spec
}

func BuildProp(prop *aproto.Prop, ctx interfaces.Context) (p *Prop, err error) {
	p = &Prop{
		Name: prop.GetName(),
	}

	if prop.GetValue() != nil {
		p.Value, err = Build(prop.GetValue(), ctx)
	}

	if prop.GetSpec() != nil {
		p.Spec, err = BuildSpec(prop.GetSpec(), ctx)
	}

	return
}

// Impl interfaces.Evaluable
func (p *Prop) Eval(ctx interfaces.Context) (unit interfaces.Unit, err error) {
	if p.Value != nil {
		unit, err = p.Value.Eval(ctx)
	} else if p.Spec != nil && p.Spec.DefaultValue != nil {
		unit, err = p.Spec.DefaultValue.Eval(ctx)
	} else {
		err = errors.New("EMPTY PROP")
	}
	return
}

// Impl interfaces.Typable
func (p *Prop) Type(ctx interfaces.Context) (t typing.Type) {
	if p.Value != nil {
		t = p.Value.Type(ctx)
	} else if p.Spec != nil {
		t = p.Spec.Type
	}

	if t == nil {
		t = typing.TypeAny
	}

	return
}

type Spec struct {
	Type         typing.Type
	DefaultValue interfaces.Unit
}

func BuildSpec(spec *aproto.PropSpec, ctx interfaces.Context) (s *Spec, err error) {
	s = &Spec{}
	if spec.GetType() != "" {
		s.Type, err = typing.Parse(spec.GetType())
	}
	if spec.GetDefaultValue() != nil {
		s.DefaultValue, err = Build(spec.GetDefaultValue(), ctx)
	}
	return
}
