package core

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/interfaces"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/typing"
)

const (
	RefPrefix = '$'
)

type Ref struct {
	name    string
	outType typing.Type
}

func BuildRef(name string, ctx interfaces.Context) (ref *Ref, err error) {
	if len(name) == 0 || name[0] != RefPrefix {
		return
	}

	name = name[1:]
	ref = &Ref{name: name}

	if val := ref.Val(ctx); val == nil {
		err = errors.REF_NOT_FOUND_ERR
	} else {
		ref.outType = val.Type(ctx)
	}

	return
}

// Impl interfaces.Evaluable
func (ref *Ref) Eval(ctx interfaces.Context) (unit interfaces.Unit, err error) {
	if val := ref.Val(ctx); val != nil {
		unit, err = val.Eval(ctx)
	} else {
		err = errors.REF_NOT_FOUND_ERR
	}
	return
}

// Impl interfaces.Typable
func (ref *Ref) Type(ctx interfaces.Context) typing.Type {
	return ctx.Get(ref.name).Type(ctx)
}

func (ref *Ref) Val(ctx interfaces.Context) interfaces.Unit {
	return ctx.Get(ref.name)
}
