package dsl

import (
	"testing"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/types"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

func TestDSL(t *testing.T) {
	a := utils.TestAsserts(t)

	makeHelloFunctorPayload := utils.FunctorPayload(&aproto.Functor{
		Name: "make_hello_msg",
		Input: &aproto.Spec{
			Spec: &aproto.Spec_Props{
				Props: &aproto.PropsSpec{Props: []*aproto.Spec{
					{
						Key: "name",
						Spec: &aproto.Spec_Props{
							Props: &aproto.PropsSpec{Props: []*aproto.Spec{
								{
									Key: "first",
									Spec: &aproto.Spec_Prop{Prop: &aproto.PropSpec{
										ValueType: "string",
									}},
								},
								{
									Key: "last",
									Spec: &aproto.Spec_Prop{Prop: &aproto.PropSpec{
										ValueType: "string",
									}},
								},
							}},
						},
					},
					{
						Key: "not_req_param",
						Spec: &aproto.Spec_Prop{Prop: &aproto.PropSpec{
							ValueType: "string",
						}},
					},
				}},
			},
		},
		Nested: []*aproto.Prop{
			{
				Key: "say_hello",
				Payload: utils.FunctorPayload(&aproto.Functor{
					Input: &aproto.Spec{
						Spec: &aproto.Spec_Prop{Prop: &aproto.PropSpec{
							ValueType: "string",
						}},
					},
					Output: &aproto.Spec{
						Spec: &aproto.Spec_Prop{Prop: &aproto.PropSpec{
							ValueType: "string",
							Value: utils.MapPayload(types.MsgBody{
								"$concat": utils.ListPayloadPayload([]*aproto.Payload{
									utils.StringPayload("Hello, "),
									utils.MapPayload(types.MsgBody{
										"$cond": utils.ListPayloadPayload([]*aproto.Payload{
											utils.MapPayload(types.MsgBody{
												"$eq": utils.ListPayloadPayload([]*aproto.Payload{
													utils.StringPayload("$input"),
													utils.StringPayload("User"),
												}),
											}),
											utils.StringPayload("$input"),
											utils.StringPayload("User"),
										}),
									}),
								}),
							}),
						}},
					},
				}),
			},
			// "say_hello": utils.MapPayload(types.MsgBody{
			// 	"$concat": utils.ListPayloadPayload([]*aproto.Payload{
			// 		utils.StringPayload("Hello, "),
			// 	}),
			// }),
		},
		Nodes: []*aproto.Prop{
			{
				Key: "say_hello_first_name",
				Payload: utils.MapPayload(types.MsgBody{
					"$say_hello": utils.StringPayload("$input.name.first"),
				}),
			},
			{
				Key: "say_hello_full_name",
				Payload: utils.MapPayload(types.MsgBody{
					"$concat": utils.ListPayloadPayload([]*aproto.Payload{
						utils.StringPayload("$say_hello_first_name"),
						utils.MapPayload(types.MsgBody{
							"$concat": utils.ListPayloadPayload([]*aproto.Payload{
								utils.StringPayload(" "),
								utils.StringPayload("$input.name.last"),
							}),
						}),
					}),
				}),
			},
		},
		Output: &aproto.Spec{
			Spec: &aproto.Spec_Prop{
				Prop: &aproto.PropSpec{
					ValueType: "string",
					Value:     utils.StringPayload("$say_hello_full_name"),
				},
			},
		},
	})
	// "sqrt_list": utils.FunctorPayload(&aproto.Functor{
	// 	Name: "sqrt_list",
	// 	Inputs: map[string]*aproto.InputSpec{
	// 		"list": {
	// 			Name:      "list",
	// 			ValueType: "int[]",
	// 		},
	// 	},
	// 	Output: &aproto.OutputSpec{
	// 		ValueType: "int[]",
	// 		Value: utils.MapPayload(types.MsgBody{
	// 			"$map": utils.MapPayload(types.MsgBody{
	// 				"func": utils.StringPayload("&sqrt"),
	// 				"list": utils.StringPayload("@list"),
	// 			}),
	// 		}),
	// 	},
	// }),

	{
		c, err := Build(makeHelloFunctorPayload)
		a.Req.NoError(err)
		a.Req.NotNil(c)

		res, err := c.Exec(utils.MapPayload(types.MsgBody{
			"name": utils.MapPayload(types.MsgBody{
				"first": utils.StringPayload("Usr"),
				"last":  utils.StringPayload("Example"),
			}),
		}))

		a.Req.NoError(err)
		a.Req.Equal("Hello, User Example", res.GetSingular().GetStringValue())
	}

	// {
	// 	res, err := object.handlers["sqrt_list"].Eval(ctx)
	// 	t.Log(res)
	// 	a.Req.NoError(err)
	// 	a.Req.NotNil(res)
	// 	a.Equal([]float64{3, 4, 5, 10}, res.GetListFloat().GetValue())
	// }
}

// func OldTestDSL(t *testing.T) {
// 	a := utils.TestAsserts(t)

// 	payload := types.MsgBody{
// 		"#def": utils.MapPayload(types.MsgBody{
// 			"make_hello_msg": utils.MapPayload(types.MsgBody{
// 				"args": utils.MapPayload(types.MsgBody{
// 					"name": utils.StringPayload("string"),
// 				}),
// 				"body": utils.MapPayload(types.MsgBody{
// 					"message": utils.MapPayload(types.MsgBody{
// 						"$concat": utils.MapPayload(types.MsgBody{
// 							"a": utils.StringPayload("Hello, "),
// 							"b": utils.StringPayload("@name"),
// 						}),
// 					}),
// 				}),
// 			}),
// 			"sqrt_list": utils.MapPayload(types.MsgBody{
// 				"args": utils.MapPayload(types.MsgBody{
// 					"list": utils.StringPayload("[]int"),
// 				}),
// 				"body": utils.MapPayload(types.MsgBody{
// 					"$map": utils.MapPayload(types.MsgBody{
// 						"func": utils.StringPayload("&sqrt"),
// 						"list": utils.StringPayload("@list"),
// 					}),
// 				}),
// 			}),
// 		}),
// 	}

// 	handler, err := BuildHandlerFromMsgData(payload)
// 	a.Req.NoError(err)
// 	a.Req.NotNil(handler)

// 	ctx := NewRuntimeFromHandler(handler)
// 	a.Req.NotNil(ctx)
// 	ctx.stack.Push(Args{
// 		"name": &Value{
// 			payload: utils.StringPayload("world"),
// 		},
// 		"list": &Value{
// 			payload: utils.ListIntPayload([]int64{
// 				9, 16, 25, 100,
// 			}),
// 		},
// 	})

// 	t.Log(handler)

// 	{
// 		res, err := handler.def["make_hello_msg"].Eval(ctx)
// 		t.Log(res)
// 		a.Req.NoError(err)
// 		a.Req.NotNil(res)
// 		a.Equal("Hello, world", res.GetMap().GetValue()["message"].GetSingular().GetStringValue())
// 	}

// 	{
// 		res, err := handler.def["sqrt_list"].Eval(ctx)
// 		t.Log(res)
// 		a.Req.NoError(err)
// 		a.Req.NotNil(res)
// 		a.Equal([]float64{3, 4, 5, 10}, res.GetListFloat().GetValue())
// 	}
// }
