package std

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/core"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/typing"
)

var (
	_ = Std.RegisterBuiltInFunc(&core.BuiltInFunctor{
		Name: "cast",
		Input: core.NewPropsSpec("", []core.Spec{
			&core.PropSpec{Key: "value", ValueType: typing.TypeAny},
			&core.PropSpec{Key: "default", ValueType: typing.TypeAny},
		}),
		OutputTypeFn: func(input core.Typable, ctx core.Context) typing.Type {
			return input.(core.Getable).Get("default").Type(ctx)
		},
		Fn: func(input core.Input, ctx core.Context) (unit core.Unit, err error) {
			fn, _ := input.Get("func").(core.Functor)

			call, _ := core.BuildClosureFunctor(fn, core.PayloadNoErr(input.Get("list"), nil), nil)
			return call.Eval(nil)
		},
	})

	_ = Std.RegisterBuiltInFunc(&core.BuiltInFunctor{
		Name: "cond",
		Input: core.NewPropsSpec("", []core.Spec{
			&core.PropSpec{Key: "if", ValueType: typing.TypeBool},
			&core.PropSpec{Key: "then", ValueType: typing.TypeAny},
			&core.PropSpec{Key: "else", ValueType: typing.TypeAny},
		}),
		OutputTypeFn: func(input core.Typable, ctx core.Context) (t typing.Type) {
			args := input.(core.Getable)
			t = args.Get("then").Type(ctx)
			if args.Get("else").Type(ctx) != t {
				t = typing.TypeBad
			}
			return t
		},
		Fn: func(input core.Input, ctx core.Context) (unit core.Unit, err error) {
			cond, err := input.Get("if").(core.Payloadable).Payload(ctx)
			if err != nil {
				return
			}

			if cond.GetSingular().GetBoolValue() {
				unit = input.Get("then")
			} else {
				unit = input.Get("else")
			}

			return
		},
	})

	_ = Std.RegisterBuiltInFunc(&core.BuiltInFunctor{
		Name: "map",
		Input: core.NewPropsSpec("", []core.Spec{
			&core.PropSpec{Key: "func", ValueType: typing.TypeAny},
			&core.PropSpec{Key: "list", ValueType: typing.TypeList},
		}),
		Output: core.NewPropsSpec("", []core.Spec{
			&core.PropSpec{Key: "list", ValueType: typing.TypeList},
		}),
		Fn: func(input core.Input, ctx core.Context) (unit core.Unit, err error) {
			fn, _ := input.Get("func").(core.Functor)

			call, _ := core.BuildClosureFunctor(fn, core.PayloadNoErr(input.Get("list"), nil), nil)
			return call.Eval(nil)
		},
	})
)
