package std

import (
	"math"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/core"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/dsl/typing"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
)

var (
	_ = Std.RegisterBuiltInFunc(&core.BuiltInFunctor{
		Name:   "sqrt",
		Input:  (&core.PropSpec{ValueType: typing.TypeScalar}),
		Output: (&core.PropSpec{ValueType: typing.TypeScalar}),
		OutputTypeFn: func(input core.Typable, ctx core.Context) (t typing.Type) {
			return input.Type(ctx)
		},
		Fn: func(input core.Input, ctx core.Context) (unit core.Unit, err error) {
			inputPayload, err := input.(core.Payloadable).Payload(ctx)
			if err != nil {
				return
			}

			var out *aproto.Payload

			switch num := inputPayload.GetSingular().GetValue().(type) {
			case *aproto.Singular_IntValue:
				out = utils.IntPayload(int64(math.Sqrt(float64(num.IntValue))))
			case *aproto.Singular_UintValue:
				out = utils.UintPayload(uint64(math.Sqrt(float64(num.UintValue))))
			case *aproto.Singular_FloatValue:
				out = utils.FloatPayload(float32(math.Sqrt(float64(num.FloatValue))))
			}

			unit = &core.UnitPayload{out}

			return
		},
	})
)
