package typing

import (
	"reflect"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

type typesCollection struct {
	byNotation   map[string]Type
	byNativeKind map[string][]Type
}

func (types *typesCollection) register(t Type) Type {
	types.byNotation[t.GetNotation()] = t
	return t
}

var (
	builtInTypes = typesCollection{
		byNotation:   make(map[string]Type),
		byNativeKind: make(map[string][]Type),
	}

	// Magic type to raise an error during type inference
	TypeBad = &CustomType{
		FnNotation: func() string { return "bad" },
		FnValidate: func(Type) bool { return false },
	}

	// Basis types
	TypeBool   = builtInTypes.register(newBasicType("bool", false, aproto.Singular_BoolValue{}))
	TypeByte   = builtInTypes.register(newBasicType("byte", byte(0), aproto.Singular_UintValue{}))
	TypeInt    = builtInTypes.register(newBasicType("int", int64(0), aproto.Singular_IntValue{}))
	TypeUint   = builtInTypes.register(newBasicType("uint", uint64(0), aproto.Singular_UintValue{}))
	TypeFloat  = builtInTypes.register(newBasicType("float", float32(0), aproto.Singular_FloatValue{}))
	TypeString = builtInTypes.register(newBasicType("string", string(""), aproto.Singular_StringValue{}))

	TypeProps     = builtInTypes.register(PropsType{})
	TypePropsList = builtInTypes.register(&ListType{NestedType: TypeProps})

	// List types
	TypeBytes = builtInTypes.register(&ListType{
		notation:   "bytes",
		NestedType: TypeByte,
		protoType:  reflect.TypeOf(aproto.Singular_BytesValue{}).Kind(),
		Shape:      []int64{-1},
	})
	TypeIVector3 = builtInTypes.register(&ListType{
		notation:   "ivector3",
		NestedType: TypeInt,
		protoType:  reflect.TypeOf(aproto.IntList{}).Kind(),
		Shape:      []int64{3},
	})
	TypeIVector4 = builtInTypes.register(&ListType{
		notation:   "ivector4",
		NestedType: TypeInt,
		protoType:  reflect.TypeOf(aproto.IntList{}).Kind(),
		Shape:      []int64{4},
	})

	// Any and set types
	TypeAny = builtInTypes.register(&CustomType{
		FnNotation: func() string { return "any" },
		FnValidate: func(Type) bool { return true },
	})

	TypeList = builtInTypes.register(&ListType{
		notation:   "list",
		NestedType: TypeAny,
		Shape:      []int64{-1},
	})

	TypeScalar = builtInTypes.register(&SetType{
		notation: "scalar",
		Types:    []Type{TypeInt, TypeUint, TypeFloat},
	})
)
