package grpc

import (
	"context"

	"google.golang.org/grpc"
	"google.golang.org/grpc/metadata"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

type Client struct {
	conn            *grpc.ClientConn
	l3ServiceClient aproto.L3ServiceClient
	inQueue         chan *aproto.TxnRequest
	outQueue        chan *aproto.TxnResponse
}

func NewClient(addr string, opts ...grpc.DialOption) (cli *Client, err error) {
	conn, err := grpc.Dial(addr, opts...)
	if err != nil {
		return
	}

	cli = &Client{
		conn:            conn,
		l3ServiceClient: aproto.NewL3ServiceClient(conn),
		inQueue:         make(chan *aproto.TxnRequest),
		outQueue:        make(chan *aproto.TxnResponse),
	}

	return
}

func (cli *Client) StartStream(clientID *proto.ID, ctx context.Context) (stream aproto.L3Service_StreamClient, err error) {
	if ctx == nil {
		ctx = context.Background()
	}
	ctx = metadata.NewOutgoingContext(
		ctx,
		metadata.New(map[string]string{
			clientIDHeaderField: clientID.Base64(),
		}),
	)

	stream, err = cli.l3ServiceClient.Stream(ctx)
	if err != nil {
		return
	}

	return
}

func (cli *Client) Close() {
	cli.conn.Close()
}
