#!python3

import json
import os
import pathlib
from typing import Any, Dict, Iterator, List, Tuple

import matplotlib.pyplot as plt

PATH = pathlib.Path(__file__).parent.resolve()
SUMMARY_PREFIX = "bench_summary"


def get_summaries() -> Iterator[Tuple[str, List[Dict[str, Any]]]]:
    return map(
        lambda f_name: (
            f_name.removesuffix(".json"),
            json.load(open(PATH / f_name, 'r'))
        ),
        filter(
            lambda f_name: f_name.startswith(
                SUMMARY_PREFIX) and f_name.endswith(".json"),
            os.listdir(PATH)
        )
    )


def plot(s_name: str, data: List[Dict[str, Any]]):
    fig, ax = plt.subplots(nrows=2, ncols=2)
    fig.set_size_inches((16, 10))

    labels = (
        (
            ("client_mean", "tab:blue"),
            ("server_mean", "tab:blue"),
        ),
        (
            ("client_std", "tab:orange"),
            ("server_std", "tab:orange"),
        )
    )
    for i, sublabels in enumerate(labels):
        for j, (label, color) in enumerate(sublabels):
            ax[i][j].scatter(
                x=list(map(lambda item: item["clients"], data)),
                y=list(map(lambda item: item[label], data)),
                color=color,
                label=label,
            )
            ax[i][j].legend()
            ax[i][j].grid(True)

            if i == 1:
                ax[i][j].set_xlabel('count of clients')
            if j == 0:
                ax[i][j].set_ylabel('time of object processing, secs')

    plt.savefig(PATH / f"{s_name}.jpeg")


def main():
    list(map(lambda args: plot(*args), get_summaries()))


if __name__ == '__main__':
    main()
