package config

import (
	"log"
	"os"
	"path"
	"strconv"
)

var (
	LOG_LEVEL = getEnv("LOG_LEVEL", "info")

	CUR_PATH, _ = os.Getwd()
	LOG_PATH    = getEnv("GRPC_SERVER_LOGS_PATH", "/tmp/grpc-server-logs")

	IIDAP_ID = getEnv("IIDAP_ID", "avvy-default-iidap")

	TICK_FRAME_RATE_S        = 0.1
	ROLLUP_MAX_TXN_COUNT     = getEnvInt("ROLLUP_MAX_TXN_COUNT", 10)
	SHUTDOWN_WAIT_TICK_COUNT = getEnvInt("SHUTDOWN_WAIT_TICK_COUNT", 100)

	//  configuration for consensus storage
	CONSENSUS_PASSWORD   = getEnv("CONSENSUS_PASSWORD", "")
	CONSENSUS_NODES_FILE = path.Join(CUR_PATH, "cluster_nodes.yml")
	CURRENT_NODE         = getEnv("CURRENT_NODE", "node1:7771:5051:54621")

	GRPC_PORT = getEnvInt("GRPC_PORT", 80)
	WS_PORT   = getEnvInt("WS_PORT", 8080)
)

func getEnv(env string, default_val string) string {
	if val, ok := os.LookupEnv(env); ok {
		return val
	}
	return default_val
}

func getEnvInt(env string, default_val int) int {
	if val, ok := os.LookupEnv(env); ok {
		val_int, err := strconv.Atoi(val)
		if err != nil {
			log.Fatalln(err)
		}
		return val_int
	}
	return default_val
}
