package exec

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/lib"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/rollup"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/store"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/subscribe"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
	"go.uber.org/zap"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

var logger = utils.GetLogger("MSG_PROCESSOR")

const (
	queueBufSize = 1000
)

type ExecController struct {
	subscribeMngr *subscribe.Manager
	store         *store.Store
	rollupProc    *rollup.RollupProcessor

	lib lang.Library

	txnsQueue chan *aproto.TxnRequest // incoming txns
	msgsQueue chan []*aproto.Msg      // internal msgs
}

func NewExecController(subscribeMngr *subscribe.Manager, st *store.Store, rollup *rollup.RollupProcessor) (exec *ExecController) {
	if subscribeMngr != nil {
		exec = &ExecController{
			subscribeMngr: subscribeMngr,
			store:         st,
			rollupProc:    rollup,
			lib:           lib.Lib,
			txnsQueue:     make(chan *aproto.TxnRequest, queueBufSize),
			msgsQueue:     make(chan []*aproto.Msg, queueBufSize),
		}
	}
	return
}

func (exec *ExecController) PushTxn(txn *aproto.TxnRequest) (err error) {
	if err = txnValidate(txn); err != nil {
		return
	}

	client := txn.AuthInfo.Signers[0]
	clientID := proto.NewID(client.PublicKey)

	logger.Debug("REGISTER TRANSACTION",
		zap.String("client_id", clientID.Base64()),
	)

	exec.txnsQueue <- txn
	return
}

func (exec *ExecController) PushMessages(msgs []*aproto.Msg) (err error) {
	exec.msgsQueue <- msgs
	return
}

func (exec *ExecController) Start() (err error) {
	batch := NewBatch()
	work := true
	var batchProcess bool
	for work {
		select {
		case txn, ok := <-exec.txnsQueue:
			if !ok {
				work = false
				break
			}
			batch.AddTxn(txn)
			batchProcess = batch.Full()
		case msgs, ok := <-exec.msgsQueue:
			if !ok {
				work = false
				break
			}
			batch.AddMessages(msgs)
			batchProcess = batch.Full()
		default:
			batchProcess = batch.Size() != 0
		}
		if batchProcess {
			batch.Process(exec)
			batch = NewBatch()
			batchProcess = false
		}
	}

	return
}

func (exec *ExecController) Stop() {
	close(exec.txnsQueue)
	close(exec.msgsQueue)
}
