package compiler

import (
	"testing"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
)

func TestParser(t *testing.T) {
	a := utils.TestAsserts(t)

	{
		prog1 := `
        {
			nums = {
				a = 111u,
				c = 555,
				b = 232f,
				d = -.12123,
			},
			a = get<string, int>(name = "b", ),
            b = "qwe",
			c = [
				123,
			],
        }`

		props, err := Compile(prog1)
		a.Req.NoError(err)
		a.Req.NotNil(props)

		a.Req.Equal(4, props.Len())

		{
			_nums := props.Get("nums").GetValue().GetProps().GetProps()
			a.Req.NotNil(_nums)
			a.Req.Len(_nums, 4)

			a.Req.Equal(uint64(111), _nums[0].GetValue().GetSingular().GetUintValue())
			a.Req.Equal(float32(232), _nums[1].GetValue().GetSingular().GetFloatValue())
			a.Req.Equal(int64(555), _nums[2].GetValue().GetSingular().GetIntValue())
			a.Req.Equal(float32(-0.12123), _nums[3].GetValue().GetSingular().GetFloatValue())
		}

		_a := props.Get("a").GetValue().GetCall()
		a.Req.NotNil(_a)
		a.Req.Equal("get", _a.Name)
		a.Req.Equal([]string{"string", "int"}, _a.Params)
		a.Req.Len(_a.Args, 1)
		a.Req.Equal("name", _a.Args[0].GetName())
		a.Req.Equal("b", _a.Args[0].GetValue().GetSingular().GetStringValue())

		_b := props.Get("b").GetValue().GetSingular().GetStringValue()
		a.Req.Equal("qwe", _b)

		_c := props.Get("c").GetValue().GetListInt().GetValue()
		a.Req.NotNil(_c)
		a.Req.Equal(int64(123), _c[0])

	}
}
