package lang

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

func EvalPayload(payload *aproto.Payload, rt *Runtime) (evaled *aproto.Payload, err error) {
	if payload == nil || rt == nil {
		return
	}

	switch val := payload.GetValue().(type) {
	case *aproto.Payload_NoEvall:
		evaled = val.NoEvall.GetPayload()
	case *aproto.Payload_Call:
		evaled, err = EvalCall(val.Call, rt)
	case *aproto.Payload_ListPayload:
		evaledList := make([]*aproto.Payload, len(val.ListPayload.GetValue()))
		for i, payload := range val.ListPayload.GetValue() {
			if evaledList[i], err = EvalPayload(payload, rt); err != nil {
				break
			}
		}
		if err == nil {
			evaled = proto.ListPayloadPayload(evaledList, nil)
		}
	default:
		evaled = payload
	}

	return
}

func EvalProps(props []*aproto.Prop, rt *Runtime) (evaled []*aproto.Prop, err error) {
	if props == nil || rt == nil {
		return
	}

	evaled = make([]*aproto.Prop, len(props))
	for i, prop := range props {
		evaled[i], err = EvalProp(prop, rt)
		if err != nil {
			break
		}
	}
	return
}

func EvalProp(prop *aproto.Prop, rt *Runtime) (evaled *aproto.Prop, err error) {
	if prop == nil || rt == nil {
		return
	}

	if err = rt.EntryToNode(prop); err != nil {
		return
	}
	defer func() { rt.ExitFromNode(prop, err) }()

	evaled = &aproto.Prop{
		Name: prop.Name,
	}
	evaled.Value, err = EvalPayload(prop.Value, rt)
	return
}

func EvalCall(call *aproto.Call, rt *Runtime) (evaled *aproto.Payload, err error) {
	if call == nil || rt == nil {
		return
	}

	// TODO: compute gas
	var gas *proto.BigUint
	defer func() {
		_err := rt.GasWithdraw(gas)
		if err == nil {
			err = _err
		}
	}()

	if fn := rt.BuildIn().Get(call.GetName()); fn != nil {
		var args []*aproto.Prop
		args, err = EvalProps(call.Args, rt)

		if err == nil {
			evaled, err = fn.Eval(args, rt)
			if err == errors.ErrNotEnoughArgs {
				err = nil
				evaled = proto.CallPayload(&aproto.Call{
					Name: call.Name,
					Args: args,
				})
			}
		}
	} else {
		err = errors.ErrFuncNotFound
	}

	return
}
