package lib

import (
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/typing"
)

var (
	_ = Lib.RegisterFn(&lang.Func{
		Name: "getIndex",
		Args: []lang.Arg{
			{
				Name: "key",
				Type: typing.TypeBytes,
			},
		},
		ReturnType: &typing.ListType{NestedType: typing.TypeBytes},
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			key := args["key"].GetValue().GetSingular().GetBytesValue()

			ids := rt.Store().Index().Get(key)
			objectIDs := make([][]byte, 0, len(ids))
			for _, id := range ids {
				objectIDs = append(objectIDs, id.Bytes())
			}

			evaled = proto.ListBytesPayload(objectIDs, nil)
			return
		},
	})

	_ = Lib.RegisterFn(&lang.Func{
		Name: "setIndex",
		Args: []lang.Arg{
			{
				Name: "key",
				Type: typing.TypeBytes,
			},
		},
		ReturnType: typing.TypeBool,
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			key := args["key"].GetValue().GetSingular().GetBytesValue()
			rt.Store().Index().Set(key, rt.Object().ID())
			evaled = proto.BoolPayload(true)
			return
		},
	})

	_ = Lib.RegisterFn(&lang.Func{
		Name: "delIndex",
		Args: []lang.Arg{
			{
				Name: "key",
				Type: typing.TypeBytes,
			},
		},
		ReturnType: typing.TypeBool,
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			key := args["key"].GetValue().GetSingular().GetBytesValue()
			rt.Store().Index().Del(key, rt.Object().ID())
			evaled = proto.BoolPayload(true)
			return
		},
	})
)
