package lib

import (
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
	"go.uber.org/zap"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/typing"
)

var (
	_ = Lib.RegisterFn(&lang.Func{
		Name:       "call",
		TypeParams: []string{"T"},
		Args: []lang.Arg{
			{
				Name: "fn",
				Type: typing.TypeFuncRef,
			},
			{
				Name: "args",
				Type: typing.TypeProps,
			},
		},
		ReturnTypeFn: func(typeParams map[string]typing.Type) typing.Type {
			return typeParams["T"]
		},
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			fnCall := args["fn"].GetValue().GetCall()
			fnCall.Args = append(fnCall.Args, args["args"].GetValue().GetProps().GetProps()...)

			evaled, err = lang.EvalCall(fnCall, rt)
			return
		},
	})

	_ = Lib.RegisterFn(&lang.Func{
		Name:       "map",
		TypeParams: []string{"INPUT_T", "OUTPUT_T"},
		Args: []lang.Arg{
			{
				Name:     "fn",
				Type:     typing.TypeFuncRef,
				SpecDesc: "Evaluable value with input referense **item** (or redefined by **key** prop)",
			},
			{
				Name: "list",
				TypeFn: func(typeParams map[string]typing.Type) typing.Type {
					return &typing.ListType{NestedType: typeParams["INPUT_T"]}
				},
				SpecDesc: "List with values, each of which will be pass to **fn**",
			},
			{
				Name:         "key",
				Type:         typing.TypeString,
				DefaultValue: proto.StringPayload("item"),
				SpecDesc:     "Arg name for passing each item of list",
			},
		},
		ReturnTypeFn: func(typeParams map[string]typing.Type) typing.Type {
			return &typing.ListType{NestedType: typeParams["OUTPUT_T"]}
		},
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			listArg := args["list"]
			keyArg := args["key"].GetValue().GetSingular().GetStringValue()

			listPayload, _ := proto.SomeListToListPayload(listArg.GetValue())

			arg := &aproto.Prop{
				Name: keyArg,
			}
			fnCall := args["fn"].GetValue().GetCall()
			fnCall.Args = append(fnCall.Args, arg)

			evaledList := make([]*aproto.Payload, len(listPayload.GetValue()))
			for i, item := range listPayload.GetValue() {
				arg.Value = item
				evaledList[i], err = lang.EvalCall(fnCall, rt)
				if err != nil {
					break
				}
			}

			if err == nil {
				evaled, _ = proto.ListPayloadToSomeList(
					&aproto.PayloadList{
						Value: evaledList,
						Shape: listPayload.GetShape(),
					},
				)
			}

			logger.Debug("STD.MAP", zap.Int("evaled_size", len(evaledList)))

			return
		},
	})
)
