package lib

import (
	"math"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/typing"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

var (
	_ = Lib.RegisterFn(&lang.Func{
		Name: "sqrt",
		Args: []lang.Arg{
			{
				Name: "num",
				Type: typing.TypeNumber,
			},
		},
		ReturnType: typing.TypeNumber,
		Fn: func(input map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			switch num := input["num"].GetValue().GetSingular().GetValue().(type) {
			case *aproto.Singular_IntValue:
				evaled = proto.IntPayload(int64(math.Sqrt(float64(num.IntValue))))
			case *aproto.Singular_UintValue:
				evaled = proto.UintPayload(uint64(math.Sqrt(float64(num.UintValue))))
			case *aproto.Singular_FloatValue:
				evaled = proto.FloatPayload(float32(math.Sqrt(float64(num.FloatValue))))
			default:
				err = errors.ErrInvalidType
			}
			return
		},
	})
)
