package lib

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/typing"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

var (
	_ = Lib.RegisterFn(&lang.Func{
		Name:       "getMsg",
		TypeParams: []string{"T"},
		Args: []lang.Arg{
			{
				Name: "name",
				Type: typing.TypeString,
			},
		},
		ReturnTypeFn: func(typeParams map[string]typing.Type) typing.Type {
			return typeParams["T"]
		},
		Fn: func(input map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {
			name := input["name"].GetValue().GetSingular().GetStringValue()
			evaled = rt.Msg().Get(name).GetValue()
			if evaled == nil {
				err = errors.ErrPropNotFound
			}
			return
		},
	})

	_ = Lib.RegisterFn(&lang.Func{
		Name: "sendMsg",
		Args: []lang.Arg{
			{
				Name: "target",
				Type: typing.TypeBytes,
			},
			{
				Name: "props",
				Type: typing.TypeProps,
			},
		},
		ReturnType: typing.TypeBool,
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {

			target := args["target"].GetValue().GetSingular().GetBytesValue()
			delete(args, "target")

			msg := &aproto.Msg{
				Target: target,
				Sender: rt.Object().ID().Bytes(),
				Props:  make([]*aproto.Prop, 0, len(args)),
			}
			msg.Props = append(msg.Props, args["props"].GetValue().GetProps().GetProps()...)

			{
				var nonce *aproto.Prop
				nonce, err = rt.Object().Get("nonce")
				if err != nil {
					return
				}
				nonce.Value = proto.UintPayload(nonce.GetValue().GetSingular().GetUintValue() + 1)
				rt.Object().Set(nonce)
			}

			rt.RegisterMsg(msg)

			evaled = proto.BoolPayload(true)
			return
		},
	})

	_ = Lib.RegisterFn(&lang.Func{
		Name: "sendEvent",
		Args: []lang.Arg{
			{
				Name: "target",
				Type: typing.TypeBytes,
			},
			{
				Name: "props",
				Type: typing.TypeProps,
			},
		},
		ReturnType: typing.TypeBool,
		Fn: func(args map[string]*aproto.Prop, rt *lang.Runtime) (evaled *aproto.Payload, err error) {

			target := args["target"].GetValue().GetSingular().GetBytesValue()
			delete(args, "target")

			msg := &aproto.Msg{
				Target: target,
				Sender: rt.Object().ID().Bytes(),
				Props:  make([]*aproto.Prop, 0, len(args)),
			}
			msg.Props = append(msg.Props, args["props"].GetValue().GetProps().GetProps()...)

			rt.RegisterEvent(msg)

			evaled = proto.BoolPayload(true)
			return
		},
	})
)
