package proto

import (
	"errors"
	"math/big"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

var (
	ErrSignedBinIntValue = errors.New("SIGNED BIN INT VALUE")
)

type BigUint struct {
	big.Int
}

func NewBigUint() *BigUint {
	return &BigUint{
		Int: *big.NewInt(0),
	}
}

func BigUintFromBytes(bts []byte) (x *BigUint, err error) {
	x = &BigUint{
		Int: *big.NewInt(0),
	}
	x.SetBytes(bts)
	if x.Sign() == -1 {
		err = ErrSignedBinIntValue
	}
	return
}

func BigUintFromUint64(num64 uint64) (x *BigUint, err error) {
	inum64 := int64(num64)
	if inum64 < 0 {
		err = ErrSignedBinIntValue
	} else {
		x = &BigUint{
			Int: *big.NewInt(inum64),
		}
	}
	return
}

func (x *BigUint) Lt(y *BigUint) bool {
	return x.Int.Cmp(&y.Int) == -1
}

func (x *BigUint) Add(y *BigUint) *BigUint {
	return &BigUint{*x.Int.Add(&x.Int, &y.Int)}
}

func (x *BigUint) Sub(y *BigUint) (res *BigUint, err error) {
	if x.Lt(y) {
		err = ErrSignedBinIntValue
	} else {
		res = &BigUint{*x.Int.Sub(&x.Int, &y.Int)}
	}
	return
}

func (x *BigUint) Payload() *aproto.Payload {
	return BytesPayload(x.Bytes())
}
