package tests

import (
	"testing"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/dapp"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/lib"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/store"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

func TestLang(t *testing.T) {
	a := utils.TestAsserts(t)

	st, err := store.MakeStore(dapp.DApp.ID().Base64(), nil)
	a.Req.NoError(err)

	{
		msg := proto.NewProps([]*aproto.Prop{
			{
				Name:  "c",
				Value: proto.IntPayload(99),
			},
		})

		object := st.Object().New()
		object.FromProps([]*aproto.Prop{
			{
				Name: "a",
				Value: proto.CallPayload(&aproto.Call{
					Name:   "getMsg",
					Params: []string{"int"},
					Args: []*aproto.Prop{
						{
							Name:  "name",
							Value: proto.StringPayload("c"),
						},
					},
				}),
			},
			{
				Name: "b",
				Value: proto.CallPayload(&aproto.Call{
					Name:   "call",
					Params: []string{"[]int"},
					Args: []*aproto.Prop{
						{
							Name: "fn",
							Value: proto.CallPayload(&aproto.Call{
								Name: "get",
								Args: []*aproto.Prop{
									{
										Name:  "name",
										Value: proto.StringPayload("ff"),
									},
								},
							}),
						},
						{
							Name: "args",
							Value: proto.PropsPayload([]*aproto.Prop{
								{
									Name: "fn",
									Value: proto.CallPayload(&aproto.Call{
										Name: "sqrt",
									}),
								},
								{
									Name:  "list",
									Value: proto.ListIntPayload([]int64{4, 9, 16}, nil),
								},
								{
									Name:  "key",
									Value: proto.StringPayload("num"),
								},
							}),
						},
					},
				}),
			},
			{
				Name: "ff",
				Value: proto.CallPayload(&aproto.Call{
					Name:   "map",
					Params: []string{"int", "int"},
				}),
			},
		})

		rt := lang.NewRuntime(store.NewBalance(nil), msg, object, st, lib.Lib)
		a.Req.NotNil(rt)

		err := rt.TypeCheck()
		a.Req.NoError(err)

		evaled, err := rt.Eval()
		a.Req.NoError(err)
		a.Req.Equal("a", evaled[0].Name)

		a.Req.Equal(int64(99), evaled[0].Value.GetSingular().GetIntValue())
		a.Req.Equal([]int64{2, 3, 4}, evaled[1].Value.GetListInt().GetValue())
		// a.Req.Equal("hello, world", evaled[2].Value.GetSingular().GetStringValue())

		// a.Req.Equal([]string{"hello, user1", "hello, user2", "hello, user3"}, evaled[4].Value.GetListString().Value)
	}

}
