package typing

import (
	"strconv"
	"strings"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/errors"
)

func Check(t1, t2 Type) (err error) {

	if !t1.Check(t2) {
		err = errors.ErrIncompatibleTypes
	}

	return
}

func Parse(nota string) (t Type, err error) {
	if len(nota) == 0 {
		err = errors.ErrInvalidTypeNotation
		return
	}

	if nota[0] == '[' {
		if endInd := strings.IndexRune(nota, ']'); endInd != -1 {
			size := -1
			if endInd > 1 {
				size, err = strconv.Atoi(nota[:endInd])
			}
			if err == nil {
				var nestedType Type
				if nestedType, err = Parse(nota[endInd+1:]); err == nil {
					listType := &ListType{
						Shape: []int64{int64(size)},
					}
					if nestedListType, isList := nestedType.(*ListType); isList {
						listType.Shape = append(listType.Shape, nestedListType.Shape...)
						listType.NestedType = nestedListType.NestedType
					} else {
						listType.NestedType = nestedType
					}
					t = listType
				}
			}
		}
	} else if t = builtInTypes.byNotation[nota]; t == nil {
		err = errors.ErrInvalidTypeNotation
	}

	return
}
