package rollup

import (
	"encoding/binary"
	"time"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/store"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

const (
	maxRollupSize    = 10
	maxRollupTimeout = 1 * time.Minute
)

var (
	logger    = utils.GetLogger("ROLLUP_PROCESSOR")
	byteOrder = binary.BigEndian
)

// TODO: IMPL RollupProcessors
type RollupProcessor struct {
	store     *store.Store
	current   *aproto.Rollup
	createdAt time.Time
}

func NewRollupProcessor(st *store.Store) *RollupProcessor {
	nonce := uint64(0)
	if lastKey := st.Rollup().ReverseIterator(nil, nil).Key(); lastKey != nil {
		nonce = byteOrder.Uint64(lastKey)
	}

	return &RollupProcessor{
		store: st,
		current: &aproto.Rollup{
			Nonce: nonce,
		},
		createdAt: time.Now().UTC(),
	}
}

func (proc *RollupProcessor) AddCycle(cycle *aproto.Rollup_Cycle) (err error) {
	proc.current.Cycles = append(proc.current.Cycles, cycle)

	if len(proc.current.Cycles) >= maxRollupSize {
		proc.rollup()
	}

	return
}

func (proc *RollupProcessor) rollup() (err error) {

	bts, err := proto.Marshal(proc.current)
	if err != nil {
		return
	}

	nonceKey := make([]byte, 0)
	byteOrder.PutUint64(nonceKey, proc.current.Nonce)

	proc.store.Rollup().Set(nonceKey, bts)
	proc.store.Commit()

	proc.current = &aproto.Rollup{
		Nonce: proc.current.Nonce + 1,
	}
	proc.createdAt = time.Now().UTC()

	return
}
