package store

import (
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
)

type Balance struct {
	orig   *proto.BigUint
	free   *proto.BigUint
	holded *proto.BigUint
}

func NewBalance(funds *proto.BigUint) *Balance {
	if funds == nil {
		funds = proto.NewBigUint()
	}
	return &Balance{
		orig:   funds,
		free:   funds,
		holded: proto.NewBigUint(),
	}
}

func (b *Balance) Withdraw(amount *proto.BigUint) (err error) {
	b.free, err = b.free.Sub(amount)
	if err != nil {
		err = errors.ErrNotEnoughBalance
	}
	return
}

func (b *Balance) Hold(amount *proto.BigUint) (err error) {
	holded := b.holded.Add(amount)

	if b.free.Lt(holded) {
		err = errors.ErrNotEnoughBalance
	} else {
		b.holded = holded
	}

	return
}

func (b *Balance) WithdrawHolded() (err error) {
	return b.Withdraw(b.holded)
}
