package store

import (
	"sync"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/errors"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
)

type registryBase struct {
	sync.RWMutex
	lockedObjects map[string]int
}

func NewRegistryBase() *registryBase {
	return &registryBase{
		RWMutex:       sync.RWMutex{},
		lockedObjects: make(map[string]int),
	}
}

func (rb *registryBase) NewRegistry(priority uint) *Registry {
	return &Registry{
		registryBase: rb,
		priority:     priority,
	}
}

type Registry struct {
	*registryBase
	priority uint
}

func (reg *Registry) check(id *proto.ID) (err error) {
	if priority, ok := reg.lockedObjects[id.Base64()]; ok && priority < int(reg.priority) {
		err = errors.ErrObjectLocked
	}
	return
}

func (reg *Registry) lock(id *proto.ID) (err error) {
	if err = reg.check(id); err == nil {
		reg.lockedObjects[id.Base64()] = int(reg.priority)
	}
	return
}
