package store

import (
	"testing"

	dbm "github.com/tendermint/tm-db"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
	"gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

func TestStore(t *testing.T) {
	a := utils.TestAsserts(t)

	db := dbm.NewMemDB()

	var objID *proto.ID

	{
		st, err := MakeStore("", db)
		a.Req.NoError(err)
		a.Req.NotNil(st)
		st, err = st.ExecWrapper(nil)
		a.Req.NoError(err)
		a.Req.NotNil(st)
		t.Log(st.store.LastCommitID())

		obj := st.Object().New()
		objID = obj.ID()
		a.Req.NotNil(obj)
		a.Req.NotNil(objID)
		obj.Set(&avvyprotocol.Prop{
			Name:  "0",
			Value: proto.StringPayload("0"),
		})
		st.Write()
		st.Commit()

		t.Log(st.store.LastCommitID())
	}

	{
		st, err := MakeStore("", db)
		a.Req.NoError(err)
		a.Req.NotNil(st)
		t.Log(st.store.LastCommitID())

		obj1, err := st.Object().Get(objID)
		a.Req.NoError(err)
		a.Req.NotNil(obj1)
		prop, err := obj1.Get("0")
		a.Req.NoError(err)
		a.Equal("0", prop.Value.GetSingular().GetStringValue())
	}

	// iter, err := db.Iterator(nil, nil)
	// a.Req.NoError(err)
	// for iter.Valid() {
	// 	t.Logf("%s : %b\n", iter.Key(), iter.Value())
	// 	iter.Next()
	// }

}
