package subscribe

import (
	"testing"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
	"gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

func TestEvent(t *testing.T) {
	a := utils.TestAsserts(t)

	mngr := DefaultManager()

	a.Req.Equal(mngr, DefaultManager())

	cliID, err := proto.MakeIDFromBase64("Zm9v")
	a.Req.NoError(err)

	a.Nil(mngr.GetClientSubscriber(cliID))
	sbr := NewSubscriber(cliID)
	mngr.RegisterSubscriber(sbr)
	sbr = mngr.GetClientSubscriber(cliID)
	a.Req.NotNil(sbr)

	mngr.BroadcastMessages([]*avvyprotocol.Msg{
		{
			Id:     1,
			Target: []byte("YmFy"),
			Sender: sbr.clientID.Bytes(),
		},
		{
			Id:     2,
			Target: sbr.clientID.Bytes(),
			Sender: []byte("YmFy"),
		},
	})

	resp, ok := sbr.RecvResponse()
	a.Req.True(ok)
	a.Len(resp.Messages, 1)
	a.Equal(int(resp.Messages[0].Id), 2)
	a.Equal(resp.Messages[0].Target, sbr.clientID.Bytes())

	mngr.CloseAllSubscribers()
	a.Nil(mngr.GetClientSubscriber(cliID))
	resp, ok = sbr.RecvResponse()
	a.Nil(resp)
	a.False(ok)
}
