package subscribe

import (
	"errors"
	"sync"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
	"go.uber.org/zap"
)

const (
	subscriberEventBufSize = 100
)

type Subscriber struct {
	mutex sync.RWMutex
	alive bool

	clientID *proto.ID
	messages chan *aproto.Msg
}

func NewSubscriber(clientID *proto.ID) *Subscriber {
	return &Subscriber{
		mutex:    sync.RWMutex{},
		alive:    true,
		clientID: clientID,
		messages: make(chan *aproto.Msg, subscriberEventBufSize),
	}
}

func (s *Subscriber) sendMsg(msg *aproto.Msg) (err error) {
	s.mutex.RLock()
	defer s.mutex.RUnlock()

	if !s.alive {
		err = errors.New("SUBSCRIBER ISN'T ALIVE")
		return
	}

	select {
	case s.messages <- msg:
		logger.Debug("Msg sended to subscriber queue",
			zap.String("client_key", s.clientID.Base64()),
			zap.Uint64("msg_id", msg.Id),
		)
	default:
		err = errors.New("SUBSCRIBER BUFFER OVERFLOW")
	}
	return
}

func (s *Subscriber) RecvResponse() (txn *aproto.TxnResponse, ok bool) {
	var msg *aproto.Msg
	msg, ok = <-s.messages
	if ok {
		txn = &aproto.TxnResponse{
			// TODO: add sign
			Messages: []*aproto.Msg{
				msg,
			},
		}
	}
	return
}

func (s *Subscriber) close() {
	s.mutex.Lock()
	defer s.mutex.Unlock()
	s.alive = false
	close(s.messages)
	logger.Debug("Subscriber closed", zap.String("client_id", s.clientID.Base64()))
}
