package tests

import (
	"testing"
	"time"

	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/exec"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/lang/proto"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/rollup"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/store"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/core/subscribe"
	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/utils"
)

func TestE2E(t *testing.T) {
	a := utils.TestAsserts(t)

	st, err := store.MakeStore("test-app", nil)
	a.Req.NoError(err)

	obj1 := st.Object().New()
	a.Req.NotNil(obj1)
	obj1.Set(&aproto.Prop{
		Name: "some_handler",
		Value: proto.CallPayload(&aproto.Call{
			Name: "sendEvent",
			Args: []*aproto.Prop{
				{
					Name:  "target",
					Value: proto.BytesPayload(obj1.ID().Bytes()),
				},
				{
					Name: "props",
					Value: proto.PropsPayload([]*aproto.Prop{
						{
							Name:  "hello",
							Value: proto.StringPayload("world"),
						},
					}),
				},
			},
		}),
	})

	commit := st.Commit()

	subMngr := subscribe.DefaultManager()
	subsriber := subscribe.NewSubscriber(obj1.ID())
	subMngr.RegisterSubscriber(subsriber)

	rlup := rollup.NewRollupProcessor(st)

	exec := exec.NewExecController(subMngr, st, rlup)

	go exec.Start()
	defer exec.Stop()

	err = exec.PushMessages([]*aproto.Msg{
		{
			Type:   "some_handler",
			Target: obj1.ID().Bytes(),
			Sender: obj1.ID().Bytes(),
		},
	})
	a.Req.NoError(err)

	time.Sleep(1 * time.Second)

	a.Req.NotEqual(commit.Version, st.LastCommitID().Version)

	txn, ok := subsriber.RecvResponse()
	a.Req.True(ok)
	a.Req.NotNil(txn)
	a.Req.Len(txn.Messages, 1)
	prop := txn.Messages[0].GetProps()[0]
	a.Req.Equal("hello", prop.Name)

}
