package utils

import (
	"log"

	"go.uber.org/zap"

	"gitlab.domi.do/avvy/avvy-l3/l3_node_golang/config"
)

var logger *zap.Logger

func LoggerSetup() {
	if logger == nil {
		var err error
		cfg := zap.NewProductionConfig()

		switch config.LOG_LEVEL {
		case "debug":
			cfg.Level = zap.NewAtomicLevelAt(zap.DebugLevel)
		case "info":
			cfg.Level = zap.NewAtomicLevelAt(zap.InfoLevel)
		case "warn":
			cfg.Level = zap.NewAtomicLevelAt(zap.WarnLevel)
		case "error":
			cfg.Level = zap.NewAtomicLevelAt(zap.ErrorLevel)
		default:
			cfg.Level = zap.NewAtomicLevelAt(zap.FatalLevel)
		}

		if logger, err = cfg.Build(); err != nil {
			log.Default().Fatalln("Fail to init zap.Logger:", err.Error())
		}
	}
}

func GetLogger(prefix string) *zap.Logger {
	if logger == nil {
		LoggerSetup()
	}
	if len(prefix) != 0 {
		return logger.Named(prefix)
	}
	return logger
}
