package proto

import (
	aproto "gitlab.domi.do/avvy/backend-protocol/proto/avvyprotocol"
)

func NullPayload() *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Null{
			Null: &aproto.Null{},
		},
	}
}

func BoolPayload(val bool) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Singular{
			&aproto.Singular{
				Value: &aproto.Singular_BoolValue{
					BoolValue: val,
				},
			},
		},
	}
}

func BytesPayload(val []byte) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Singular{
			&aproto.Singular{
				Value: &aproto.Singular_BytesValue{
					BytesValue: val,
				},
			},
		},
	}
}

func IntPayload(val int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Singular{
			&aproto.Singular{
				Value: &aproto.Singular_IntValue{
					IntValue: val,
				},
			},
		},
	}
}

func UintPayload(val uint64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Singular{
			&aproto.Singular{
				Value: &aproto.Singular_UintValue{
					UintValue: val,
				},
			},
		},
	}
}

func FloatPayload(val float32) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Singular{
			&aproto.Singular{
				Value: &aproto.Singular_FloatValue{
					FloatValue: val,
				},
			},
		},
	}
}

func StringPayload(val string) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Singular{
			&aproto.Singular{
				Value: &aproto.Singular_StringValue{
					StringValue: val,
				},
			},
		},
	}
}

func ListBoolPayload(val []bool, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListBool{
			ListBool: &aproto.BoolList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func ListBytesPayload(val [][]byte, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListBytes{
			ListBytes: &aproto.BytesList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func ListIntPayload(val []int64, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListInt{
			ListInt: &aproto.IntList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func ListUintPayload(val []uint64, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListUint{
			ListUint: &aproto.UintList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func ListFloatPayload(val []float32, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListFloat{
			ListFloat: &aproto.FloatList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func ListStringPayload(val []string, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListString{
			ListString: &aproto.StringList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func ListPayloadPayload(val []*aproto.Payload, shape []int64) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_ListPayload{
			ListPayload: &aproto.PayloadList{
				Value: val,
				Shape: shape,
			},
		},
	}
}

func PropsPayload(props []*aproto.Prop) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Props{
			Props: &aproto.PropList{
				Props: props,
			},
		},
	}
}

func CallPayload(call *aproto.Call) *aproto.Payload {
	return &aproto.Payload{
		Value: &aproto.Payload_Call{
			Call: call,
		},
	}
}

func SomeListToListPayload(inList *aproto.Payload) (outList *aproto.PayloadList, ok bool) {
	payloadList := make([]*aproto.Payload, 0)
	var shape []int64

	ok = true

	switch value := inList.GetValue().(type) {
	case *aproto.Payload_ListBool:
		list := value.ListBool
		shape = list.GetShape()
		for _, val := range list.GetValue() {
			payloadList = append(payloadList, BoolPayload(val))
		}
	case *aproto.Payload_ListBytes:
		list := value.ListBytes
		shape = list.GetShape()
		for _, val := range list.GetValue() {
			payloadList = append(payloadList, BytesPayload(val))
		}
	case *aproto.Payload_ListInt:
		list := value.ListInt
		shape = list.GetShape()
		for _, val := range list.GetValue() {
			payloadList = append(payloadList, IntPayload(val))
		}
	case *aproto.Payload_ListUint:
		list := value.ListUint
		shape = list.GetShape()
		for _, val := range list.GetValue() {
			payloadList = append(payloadList, UintPayload(val))
		}
	case *aproto.Payload_ListFloat:
		list := value.ListFloat
		shape = list.GetShape()
		for _, val := range list.GetValue() {
			payloadList = append(payloadList, FloatPayload(val))
		}
	case *aproto.Payload_ListString:
		list := value.ListString
		shape = list.GetShape()
		for _, val := range list.GetValue() {
			payloadList = append(payloadList, StringPayload(val))
		}
	case *aproto.Payload_ListPayload:
		payloadList = value.ListPayload.GetValue()
		shape = value.ListPayload.GetShape()
	default:
		ok = false
	}

	outList = &aproto.PayloadList{
		Value: payloadList,
		Shape: shape,
	}

	return
}

func ListPayloadToSomeList(inList *aproto.PayloadList) (payload *aproto.Payload, ok bool) {
	payloadList := inList.GetValue()
	shape := inList.GetShape()
	if len(payloadList) == 0 || payloadList[0].GetSingular() == nil {
		return
	}

	switch payloadList[0].GetSingular().GetValue().(type) {
	case *aproto.Singular_BoolValue:
		list := make([]bool, 0, len(payloadList))
		for i := range payloadList {
			if sing := payloadList[i].GetSingular(); sing != nil {
				list = append(list, sing.GetBoolValue())
			} else {
				return
			}
		}
		payload = ListBoolPayload(list, shape)
	case *aproto.Singular_BytesValue:
		list := make([][]byte, 0, len(payloadList))
		for i := range payloadList {
			if sing := payloadList[i].GetSingular(); sing != nil {
				list = append(list, sing.GetBytesValue())
			} else {
				return
			}
		}
		payload = ListBytesPayload(list, shape)
	case *aproto.Singular_IntValue:
		list := make([]int64, 0, len(payloadList))
		for i := range payloadList {
			if sing := payloadList[i].GetSingular(); sing != nil {
				list = append(list, sing.GetIntValue())
			} else {
				return
			}
		}
		payload = ListIntPayload(list, shape)
	case *aproto.Singular_UintValue:
		list := make([]uint64, 0, len(payloadList))
		for i := range payloadList {
			if sing := payloadList[i].GetSingular(); sing != nil {
				list = append(list, sing.GetUintValue())
			} else {
				return
			}
		}
		payload = ListUintPayload(list, shape)
	case *aproto.Singular_FloatValue:
		list := make([]float32, 0, len(payloadList))
		for i := range payloadList {
			if sing := payloadList[i].GetSingular(); sing != nil {
				list = append(list, sing.GetFloatValue())
			} else {
				return
			}
		}
		payload = ListFloatPayload(list, shape)
	case *aproto.Singular_StringValue:
		list := make([]string, 0, len(payloadList))
		for i := range payloadList {
			if sing := payloadList[i].GetSingular(); sing != nil {
				list = append(list, sing.GetStringValue())
			} else {
				return
			}
		}
		payload = ListStringPayload(list, shape)
	}

	ok = true

	return
}
